package miolnet.guconnectivity;

import java.io.IOException;
import java.util.Map;

/**
 * This interface defines methods for connecting to GatheringUnits and getting the plugged 
 * sensors and it's values. GatheringUnits are devices to which are connected sensors DS18B20 over 1-wire bus.
 * 
 * @author milool
 *
 */
public interface GUConnectivity {

	/**
	 * Use this method to connect to GatheringUnit. Returns true if connected or some exception if some 
	 * problem occured. Returns false if some of the parameters is null.
	 * 
	 * @param ip The IP address of GatheringUnit.
	 * @param username The name of the user used to get the data.
	 * @param password The user's password.
	 * @return Returns true if connected successfully or false, if not.
	 * @throws IOException
	 * @throws AuthenticationFailedException
	 */
	boolean connect(final String ip, final String username, final String password) throws IOException, AuthenticationFailedException;
	
	/**
	 * This method closes the connection.
	 * 
	 * @throws IOException
	 */
	void close() throws IOException;
	
	/**
	 * This method gets data from GatheringUnit's sensors as map of USC, Temp. You must call the connect method 
	 * first! Also don't forget to close the connection. 
	 * 
	 * @return Returns map of USC, Temp.
	 * @throws IOException
	 */
	Map<String, Float> gatherTemperatures() throws IOException;
	
}
