package miolnet.model;

import java.io.Serializable;
import java.util.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

@Entity
@Table(name = "Gatheringunits")
@NamedQueries({
	@NamedQuery(name = "findAllGU", query = "select g from GatheringUnit g")
})
public class GatheringUnit implements Serializable {
	
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue
	@Column(name = "GatheringunitID")
	private int id;
	
	@NotNull
	@Size(max = 60)
	@Column(name = "Name", nullable = false)
	private String name;
	
	@NotNull
	@Size(max = 39)
	@Column(name = "Ip", nullable = false)
	private String ip;
	
	@Size(max = 65535)
	@Column(name = "Description")
	private String description;
	
	@NotNull
	@OneToOne(cascade = CascadeType.ALL)
	@JoinColumn(name = "LogindetailsID", nullable = false)
	private LoginDetails loginDetails;
	
	@OneToMany(mappedBy = "gatheringUnit", fetch = FetchType.EAGER)
	private Set<Sensor> sensors;
	
	@Column(name = "Available")
	private boolean available;
	
	@Column(name = "LastGatherTime")
	private Date lastGatherTime;

	public GatheringUnit() {
	}
	
	public GatheringUnit(String name, String ip, String description, LoginDetails loginDetails) {
		super();
		this.name = name;
		this.ip = ip;
		this.description = description;
		this.loginDetails = loginDetails;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getIp() {
		return ip;
	}

	public void setIp(String ip) {
		this.ip = ip;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public LoginDetails getLoginDetails() {
		return loginDetails;
	}

	public void setLoginDetails(LoginDetails loginDetails) {
		this.loginDetails = loginDetails;
	}

	public Set<Sensor> getSensors() {
		return sensors;
	}

	public void setSensors(Set<Sensor> sensors) {
		this.sensors = sensors;
	}
	
	public boolean isAvailable() {
		return available;
	}

	public void setAvailable(boolean available) {
		this.available = available;
	}

	public Date getLastGatherTime() {
		return lastGatherTime;
	}

	public void setLastGatherTime(Date lastGatherTime) {
		this.lastGatherTime = lastGatherTime;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		GatheringUnit other = (GatheringUnit) obj;
		return id == other.id;
	}

	@Override
	public String toString() {
		return "GatheringUnit [id=" + id + ", name=" + name + "]";
	}
	
}
