package miolnet.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

@Entity
@Table(name = "Modules")
@NamedQueries({
	@NamedQuery(name = "findModuleByTag", query = "select m from Module m where m.tag = :tag"),
	@NamedQuery(name = "findAllModules", query = "select m from Module m")
})
public class Module implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue
	@Column(name = "ModuleID")
	private int id;
	
	@NotNull
	@Size(max = 40)
	@Column(name = "Name", nullable = false)
	private String name;
	
	@Column(name = "Enabled")
	private boolean enabled;
	
	@Size(max = 40)
	@Column(name = "Author")
	private String author;
	
	@NotNull
	@Size(max = 20)
	@Column(name = "Tag", nullable = false, unique = true)
	private String tag;
	
	@Size(max = 12)
	@Column(name = "Version")
	private String version;
	
	@Size(max = 65535)
	@Column(name = "Description")
	private String description;
	
	@OneToMany(mappedBy = "module", fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
	private List<Setting> settings;
	
	@OneToMany(mappedBy = "module", fetch = FetchType.EAGER, cascade = CascadeType.ALL, orphanRemoval = true)
	private List<ModuleSensor> moduleSensors;
	
	public Module() {
		settings = new ArrayList<>();
		moduleSensors = new ArrayList<>();
	}

	public Module(String name, String tag) {
		this.name = name;
		this.tag = tag;
		settings = new ArrayList<>();
		moduleSensors = new ArrayList<>();
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public boolean isEnabled() {
		return enabled;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public String getTag() {
		return tag;
	}

	public void setTag(String tag) {
		this.tag = tag;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public List<Setting> getSettings() {
		return settings;
	}

	public void setSettings(List<Setting> settings) {
		this.settings = settings;
	}

	public List<ModuleSensor> getModuleSensors() {
		return moduleSensors;
	}

	public void setModuleSensors(List<ModuleSensor> moduleSensors) {
		this.moduleSensors = moduleSensors;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Module other = (Module) obj;
		return id == other.id;
	}

	@Override
	public String toString() {
		return "Module [id=" + id + ", name=" + name + ", enabled=" + enabled + ", author=" + author + ", tag=" + tag
				+ ", version=" + version + "]";
	}
	
}
