package miolnet.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

@Entity
@Table(name = "Records")
@NamedQueries({
	@NamedQuery(name = "findRecords", query = "select r from Record r where r.value like :needle order by r.recordtime DESC"),
	@NamedQuery(name = "findUserRecords", query = "select r from Record r where r.user.id = :userId and r.value like :needle order by r.recordtime DESC"),
	@NamedQuery(name = "findModuleRecords", query = "select r from Record r where r.module.id = :moduleId and r.value like :needle order by r.recordtime DESC"),
	@NamedQuery(name = "findRecordsInInterval", query = "select r from Record r where r.value like :needle and r.recordtime between :from and :to order by r.recordtime ASC"),
	@NamedQuery(name = "findUserRecordsInInterval", query = "select r from Record r where r.user.id = :userId and r.value like :needle and r.recordtime between :from and :to order by r.recordtime ASC"),
	@NamedQuery(name = "findModuleRecordsInInterval", query = "select r from Record r where r.module.id = :moduleId and r.value like :needle and r.recordtime between :from and :to order by r.recordtime ASC")
})
public class Record implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue
	@Column(name = "RecordID")
	private int id;
	
	@NotNull
	@Column(name = "Recordtime", nullable = false)
	private Date recordtime;
	
	private boolean status;
	
	@NotNull
	@Size(max = 128)
	@Column(name = "Value", nullable = false)
	private String value;
	
	@ManyToOne
	@JoinColumn(name = "UserID", updatable = false)
	private User user;
	
	@ManyToOne
	@JoinColumn(name = "ModuleID", updatable = false)
	private Module module;
	
	public Record() {
	}

	public Record(Date recordtime, String value, boolean status) {
		this.recordtime = recordtime;
		this.status = status;
		this.value = value;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public Date getRecordtime() {
		return recordtime;
	}

	public void setRecordtime(Date recordtime) {
		this.recordtime = recordtime;
	}

	public boolean isStatus() {
		return status;
	}

	public void setStatus(boolean status) {
		this.status = status;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public Module getModule() {
		return module;
	}

	public void setModule(Module module) {
		this.module = module;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Record other = (Record) obj;
		return id == other.id;
	}

	@Override
	public String toString() {
		return "Record [id=" + id + ", recordtime=" + recordtime + ", status=" + status + ", value=" + value + ", user="
				+ user + ", module=" + module + "]";
	}

}
