package miolnet.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

@Entity
@Table(name = "Sensors")
@NamedQueries({
	@NamedQuery(name = "findAllSensors", query = "select g from Sensor g"),
	@NamedQuery(name = "findSensorByUsc", query = "select g from Sensor g where g.usc = :usc"),
	@NamedQuery(name = "findSensorsWithoutLGT", query = "select g from Sensor g where g.lastGatherTime < :lastGatherTime")
})
public class Sensor implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue
	@Column(name = "SensorID")
	private int id;
	
	@NotNull
	@Size(max = 15)
	@Column(name = "Usc", nullable = false, updatable = false, unique = true)
	private String usc;
	
	@NotNull
	@Size(max = 60)
	@Column(name = "Name", nullable = false)
	private String name;
	
	@Size(max = 65535)
	@Column(name = "Notes")
	private String notes;
	
	@Column(name = "Installed", nullable = false, updatable = false)
	private Date installed;

	@Column(name = "Available")
	private boolean available;
	
	@Column(name = "Calibration")
	private float calibration;
	
	@Column(name = "GatherInterval")
	private int interval;
	
	@Column(name = "LastGatherTime")
	private Date lastGatherTime;
	
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "GatheringunitID", nullable = false)
	private GatheringUnit gatheringUnit;
	
	public Sensor() {
	}

	public Sensor(String usc, String name, Date installed, boolean available, GatheringUnit gatheringUnit) {
		super();
		this.usc = usc;
		this.name = name;
		this.installed = installed;
		this.available = available;
		this.gatheringUnit = gatheringUnit;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getUsc() {
		return usc;
	}

	public void setUsc(String usc) {
		this.usc = usc;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getNotes() {
		return notes;
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}

	public Date getInstalled() {
		return installed;
	}

	public void setInstalled(Date installed) {
		this.installed = installed;
	}

	public boolean isAvailable() {
		return available;
	}

	public void setAvailable(boolean available) {
		this.available = available;
	}

	public float getCalibration() {
		return calibration;
	}

	public void setCalibration(float calibration) {
		this.calibration = calibration;
	}

	public int getInterval() {
		return interval;
	}

	public void setInterval(int interval) {
		this.interval = interval;
	}

	public GatheringUnit getGatheringUnit() {
		return gatheringUnit;
	}

	public void setGatheringUnit(GatheringUnit gatheringUnit) {
		this.gatheringUnit = gatheringUnit;
	}

	public Date getLastGatherTime() {
		return lastGatherTime;
	}

	public void setLastGatherTime(Date lastGatherTime) {
		this.lastGatherTime = lastGatherTime;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((usc == null) ? 0 : usc.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Sensor other = (Sensor) obj;
		if (usc == null && other.usc != null) {
			return false;
		}
		return usc.equals(other.usc);
	}

	@Override
	public String toString() {
		return "Sensor [id=" + id + ", usc=" + usc + ", name=" + name + "]";
	}
	
}
