package miolnet.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

@Entity
@Table(name = "Settings")
@NamedQueries({
	@NamedQuery(name = "findAllCoreSettings", query = "select s from Setting s where s.module is null"),
	@NamedQuery(name = "findSettingsByKey", query = "select s from Setting s where s.name = :name")
})
public class Setting implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue
	@Column(name = "SettingID")
	private int id;
	
	@NotNull
	@Size(min = 1, max = 32)
	@Column(name = "Name")
	private String name;
	
	@Size(max = 128)
	@Column(name = "Value")
	private String value;
	
	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ModuleID")
	private Module module;
	
	public Setting() {
	}

	public Setting(String name, String value) {
		this.name = name;
		this.value = value;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public Module getModule() {
		return module;
	}

	public void setModule(Module module) {
		this.module = module;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Setting other = (Setting) obj;
		return id == other.id;
	}

	@Override
	public String toString() {
		return "Setting [id=" + id + ", name=" + name + ", value=" + value + ", module=" + module + "]";
	}
	
}
