package miolnet.model;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

@Entity
@Table(name = "Temperatures")
@NamedQueries({
	@NamedQuery(name = "findSensorLastTemps", query = "select t from Temperature t where t.sensor.id = :sensorId order by t.gather.gatherTime DESC"),
	@NamedQuery(name = "findSensorIntervalTemps", query = "select t from Temperature t where t.sensor.id = :sensorId and t.gather.gatherTime between :from and :to order by t.gather.gatherTime DESC"),
	@NamedQuery(name = "avgSensorIntervalTemp", query = "select avg(t.temp) from Temperature t where t.sensor.id = :sensorId and t.gather.gatherTime between :from and :to order by t.gather.gatherTime DESC"),
	@NamedQuery(name = "maxSensorIntervalVal", query = "select max(t.temp) from Temperature t where t.sensor.id = :sensorId and t.gather.gatherTime between :from and :to order by t.gather.gatherTime DESC"),
	@NamedQuery(name = "minSensorIntervalVal", query = "select min(t.temp) from Temperature t where t.sensor.id = :sensorId and t.gather.gatherTime between :from and :to order by t.gather.gatherTime DESC"),
	@NamedQuery(name = "findSensorIntervalTemp", query = "select t from Temperature t where t.sensor.id = :sensorId and t.temp = :temp and t.gather.gatherTime between :from and :to order by t.gather.gatherTime DESC")
})
public class Temperature implements Serializable {

	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue
	@Column(name = "TemperatureID")
	private int id;
	
	@NotNull
	@Max(value=70)
	@Min(value=-50)
	@Column(name = "Temp", nullable = false, updatable = false)
	private float temp;
	
	@NotNull
	@ManyToOne(cascade = CascadeType.PERSIST)
	@JoinColumn(name = "GatherID", nullable = false, updatable = false)
	private Gather gather;
	
	@NotNull
	@ManyToOne
	@JoinColumn(name = "SensorID", nullable = false, updatable = false)
	private Sensor sensor;
	
	public Temperature() {
	}

	public Temperature(float temp, Gather gather, Sensor sensor) {
		super();
		this.temp = temp;
		this.gather = gather;
		this.sensor = sensor;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public float getTemp() {
		return temp;
	}

	public void setTemp(float temp) {
		this.temp = temp;
	}

	public Gather getGather() {
		return gather;
	}

	public void setGather(Gather gather) {
		this.gather = gather;
	}

	public Sensor getSensor() {
		return sensor;
	}

	public void setSensor(Sensor sensor) {
		this.sensor = sensor;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + id;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Temperature other = (Temperature) obj;
		return id == other.id;
	}

	@Override
	public String toString() {
		return "Temperature [id=" + id + ", temp=" + temp + ", gather=" + gather + ", sensor=" + sensor + "]";
	}

}
