package miolnet.services;

import java.util.List;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import miolnet.guconnectivity.Protocol;
import miolnet.model.GatheringUnit;
import miolnet.model.LoginDetails;

/**
 * Session Bean implementation class GatheringUnitService
 */
@Stateless
public class GatheringUnitService implements GatheringUnitServiceLocal {

	@Inject
	private EntityManager em;
	
    /**
     * Default constructor. 
     */
    public GatheringUnitService() {
    }

	@Override
	public GatheringUnit createGatheringUnit(final GatheringUnit gu) {
		em.persist(gu);
		return gu;
	}

	@Override
	public GatheringUnit createGatheringUnit(String ip, String username, String password, Protocol protocol) {
		final GatheringUnit gu = new GatheringUnit("", ip, "", new LoginDetails(username, password, protocol));
		em.persist(gu);
		return gu;
	}

	@Override
	public GatheringUnit updateGatheringUnit(final GatheringUnit gu) {
		return em.merge(gu);
	}

	@Override
	public List<GatheringUnit> getAll() {
		TypedQuery<GatheringUnit> query = em.createNamedQuery("findAllGU", GatheringUnit.class);
		return query.getResultList();
	}

	@Override
	public GatheringUnit getById(int id) {
		return em.find(GatheringUnit.class, id);
	}

	@Override
	public void removeGatheringUnit(final GatheringUnit gu) {
		final GatheringUnit gunit = em.find(GatheringUnit.class, gu.getId());
		em.remove(gunit);
	}
    
}
