package miolnet.services;

import java.util.List;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;

import miolnet.model.Module;

/**
 * Session Bean implementation class ModuleService
 */
@Stateless
public class ModuleService implements ModuleServiceLocal {
	
	@Inject
	private EntityManager em;

    /**
     * Default constructor. 
     */
    public ModuleService() {
    }

	@Override
	public Module createModule(final Module module) {
		em.persist(module);
		return module;
	}

	@Override
	public Module createModule(final String name, final String tag, final String author, final String version) {
		final Module module = new Module(name, tag);
		module.setAuthor(author);
		module.setVersion(version);
		em.persist(module);
		return module;
	}

	@Override
	public Module updateModule(final Module module) {
		return em.merge(module);
	}

	@Override
	public Module removeModule(final Module module) {
		final Module m = em.find(Module.class, module.getId());
		em.remove(m);
		return m;
	}

	@Override
	public List<Module> getAll() {
		final TypedQuery<Module> query = em.createNamedQuery("findAllModules", Module.class);
		return query.getResultList();
	}

	@Override
	public Module getById(int id) {
		return em.find(Module.class, id);
	}

	@Override
	public Module getByTag(final String tag) {
		final TypedQuery<Module> query = em.createNamedQuery("findModuleByTag", Module.class);
		query.setParameter("tag", tag);
		try {
			return query.getSingleResult();
		} catch (NoResultException e) {
		}
		return null;
	}

}
