package miolnet.services;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import miolnet.model.Module;
import miolnet.model.Record;
import miolnet.model.User;

/**
 * Session Bean implementation class RecordService
 */
@Stateless
public class RecordService implements RecordServiceLocal {

	@Inject
	private EntityManager em;
	
    /**
     * Default constructor. 
     */
    public RecordService() {
        // TODO Auto-generated constructor stub
    }

	@Override
	public Record createRecord(final Record record) {
		em.persist(record);
		return record;
	}

	@Override
	public Record createRecord(final Date recordTime, final String value, boolean status) {
		final Record r = new Record(recordTime, value, status);
		em.persist(r);
		return r;
	}

	@Override
	public Record createRecord(final Date recordTime, final String value, boolean status, final User user) {
		final Record r = new Record(recordTime, value, status);
		r.setUser(user);
		em.persist(r);
		return r;
	}

	@Override
	public Record createRecord(final Date recordTime, final String value, boolean status, final Module module) {
		final Record r = new Record(recordTime, value, status);
		r.setModule(module);
		em.persist(r);
		return r;
	}

	@Override
	public List<Record> getLastNRecordsWithValue(final String needle, int n) {
		final TypedQuery<Record> query = em.createNamedQuery("findRecords", Record.class);
		query.setParameter("needle", "%"+needle+"%");
		query.setMaxResults(n);
		final List<Record> records = query.getResultList();
		Collections.reverse(records);
		return records;
	}

	@Override
	public List<Record> getLastNRecordsWithValue(final User user, final String needle, int n) {
		final TypedQuery<Record> query = em.createNamedQuery("findUserRecords", Record.class);
		query.setParameter("userId", user.getId());
		query.setParameter("needle", "%"+needle+"%");
		query.setMaxResults(n);
		final List<Record> records = query.getResultList();
		Collections.reverse(records);
		return records;
	}

	@Override
	public List<Record> getLastNRecordsWithValue(final Module module, final String needle, int n) {
		final TypedQuery<Record> query = em.createNamedQuery("findModuleRecords", Record.class);
		query.setParameter("moduleId", module.getId());
		query.setParameter("needle", "%"+needle+"%");
		query.setMaxResults(n);
		final List<Record> records = query.getResultList();
		Collections.reverse(records);
		return records;
	}

	@Override
	public List<Record> getRecordsWithValueInDateInterval(final String needle, final Date start, final Date end) {
		final TypedQuery<Record> query = em.createNamedQuery("findRecordsInInterval", Record.class);
		query.setParameter("needle", "%"+needle+"%");
		query.setParameter("from", start);
		query.setParameter("to", end);
		return query.getResultList();
	}

	@Override
	public List<Record> getRecordsWithValueInDateInterval(final User user, final String needle, final Date start, final Date end) {
		final TypedQuery<Record> query = em.createNamedQuery("findUserRecordsInInterval", Record.class);
		query.setParameter("userId", user.getId());
		query.setParameter("needle", "%"+needle+"%");
		query.setParameter("from", start);
		query.setParameter("to", end);
		return query.getResultList();
	}

	@Override
	public List<Record> getRecordsWithValueInDateInterval(final Module module, final String needle, final Date start, final Date end) {
		final TypedQuery<Record> query = em.createNamedQuery("findModuleRecordsInInterval", Record.class);
		query.setParameter("moduleId", module.getId());
		query.setParameter("needle", "%"+needle+"%");
		query.setParameter("from", start);
		query.setParameter("to", end);
		return query.getResultList();
	}

}
