package miolnet.services;

import java.util.Date;
import java.util.List;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;

import miolnet.model.GatheringUnit;
import miolnet.model.Sensor;

/**
 * Session Bean implementation class SensorService
 */
@Stateless
public class SensorService implements SensorServiceLocal {

	@Inject
	private EntityManager em;
	
	@Inject
	private GatheringUnitServiceLocal gUnitService;
	
    /**
     * Default constructor. 
     */
    public SensorService() {
    }

	@Override
	public Sensor createSensor(final Sensor sensor) {
		em.persist(sensor);
		return sensor;
	}

	@Override
	public Sensor createSensor(final String usc, final GatheringUnit gu) {
		final Sensor sensor = new Sensor(usc, usc, new Date(), false, gu);
		gu.getSensors().add(sensor);
		em.persist(sensor);
		gUnitService.updateGatheringUnit(gu);
		return sensor;
	}

	@Override
	public Sensor updateSensor(final Sensor sensor) {
		return em.merge(sensor);
	}

	@Override
	public List<Sensor> getAll() {
		TypedQuery<Sensor> query = em.createNamedQuery("findAllSensors", Sensor.class);
		return query.getResultList();
	}

	@Override
	public Sensor getById(int id) {
		return em.find(Sensor.class, id);
	}

	@Override
	public void removeSensor(final Sensor sensor) {
		final Sensor s = em.find(Sensor.class, sensor.getId());
		em.remove(s);
	}

	@Override
	public Sensor getByUsc(String usc) {
		TypedQuery<Sensor> query = em.createNamedQuery("findSensorByUsc", Sensor.class);
		query.setParameter("usc", usc);
		try {
			return query.getSingleResult();
		} catch(NoResultException e) {
		}
		return null;
	}

	@Override
	public List<Sensor> getAllWithoutThisLastGatherTime(Date lastGatherTime) {
		TypedQuery<Sensor> query = em.createNamedQuery("findSensorsWithoutLGT", Sensor.class);
		query.setParameter("lastGatherTime", lastGatherTime);
		return query.getResultList();
	}

}
