package miolnet.services;

import java.util.List;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import miolnet.model.Setting;

/**
 * Session Bean implementation class SettingService
 */
@Stateless
public class SettingService implements SettingServiceLocal {

	@Inject
	private EntityManager em;
	
    /**
     * Default constructor. 
     */
    public SettingService() {
        // TODO Auto-generated constructor stub
    }

	@Override
	public Setting createSetting(final Setting setting) {
		em.persist(setting);
		return setting;
	}

	@Override
	public Setting createSetting(final String name, final String value) {
		final Setting setting = new Setting(name, value);
		em.persist(setting);
		return setting;
	}

	@Override
	public Setting updateSetting(final Setting setting) {
		return em.merge(setting);
	}

	@Override
	public Setting removeSetting(final Setting setting) {
		final Setting s = em.find(Setting.class, setting.getId());
		em.remove(s);
		return s;
	}

	@Override
	public List<Setting> getSettingsByName(final String name) {
		final TypedQuery<Setting> query = em.createNamedQuery("findSettingsByKey", Setting.class);
		query.setParameter("name", name);
		return query.getResultList();
	}

	@Override
	public List<Setting> getCoreSettings() {
		final TypedQuery<Setting> query = em.createNamedQuery("findAllCoreSettings", Setting.class);
		return query.getResultList();
	}

	@Override
	public Setting getSettingById(int id) {
		return em.find(Setting.class, id);
	}

}
