package miolnet.services;

import java.util.List;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;

import miolnet.model.Role;
import miolnet.model.User;
import miolnet.tools.Encryption;

/**
 * Session Bean implementation class UserService
 */
@Stateless
public class UserService implements UserServiceLocal {

	@Inject
	private EntityManager em;
	
    /**
     * Default constructor. 
     */
    public UserService() {
        // TODO Auto-generated constructor stub
    }

	@Override
	public User createUser(final User user) {
		em.persist(user);
		return user;
	}

	@Override
	public User createUser(final String username, final String password, final Role role) {
		final User user = new User(username, Encryption.generateSha256Hash(password), role);
		em.persist(user);
		return user;
	}

	@Override
	public User updateUser(final User user) {
		final User u = getById(user.getId());
		if (u != null && !u.getPassword().equals(user.getPassword())) {
			if (user.getPassword() == null || user.getPassword().isEmpty()) {
				user.setPassword(u.getPassword());
			}
			else {
				user.setPassword(Encryption.generateSha256Hash(user.getPassword()));
			}
		}
		return em.merge(user);
	}

	@Override
	public User removeUser(final User user) {
		final User u = getById(user.getId());
		em.remove(u);
		return u;
	}

	@Override
	public List<User> getAll() {
		final TypedQuery<User> query = em.createNamedQuery("findAllUsers", User.class);
		return query.getResultList();
	}

	@Override
	public User getById(int id) {
		return em.find(User.class, id);
	}

	@Override
	public User getByUsername(final String username) {
		final TypedQuery<User> query = em.createNamedQuery("findUserByName", User.class);
		query.setParameter("name", username);
		try {
			return query.getSingleResult();
		} catch(NoResultException e) {
		}
		return null;
	}

}
