package miolnet.modules.heating;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.ejb.Singleton;
import javax.ejb.Startup;

import miolnet.model.Module;
import miolnet.model.Setting;
import miolnet.services.ModuleServiceLocal;

@Singleton
@Startup
public class HeatingModule {
	
	private static final Logger LOGGER = Logger.getLogger(HeatingModule.class.getName());
	
	public static final String NAME = "Vytápění";
	public static final String TAG = "heating";
	public static final String AUTHOR = "Olmr Miloš";
	public static final String VERSION = "v0.1";
	
	private static final Map<String, String> initConfig = new HashMap<>();
	
	public static float DISPLAY_TEMP = 0;

	@EJB
	private ModuleServiceLocal moduleService;
	
    public HeatingModule() {
    	//initConfig.put("heatingOutdoorSensorUSC", "");
    	initConfig.put("heatingHysteresisDifference", "1");
    	initConfig.put("heatingEnabled", "false");
    	initConfig.put("heatingHeatNow", "false");
    	initConfig.put("heatingHeatNowDuration", "30");
    	initConfig.put("heatingOutdoorTempLimit", "13");
    }
    
    @PostConstruct
    public void init() {
    	//create module if not exist
    	Module module = moduleService.getByTag(TAG);
    	if (module==null) {
    		module = moduleService.createModule(NAME, TAG, AUTHOR, VERSION);
    		module.setEnabled(true);
    	}
    	
    	//add not existing config entries
    	final Set<String> keys = new HashSet<>();
		if (!module.getSettings().isEmpty()) {
			for (Setting s : module.getSettings()) {
				keys.add(s.getName());
			}
		}
		Setting s;
		final Iterator<Entry<String, String>> it = initConfig.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, String> entry = it.next();
			if (!keys.contains(entry.getKey())) {
				s = new Setting(entry.getKey(), entry.getValue());
				s.setModule(module);
				module.getSettings().add(s);
			}
		}
		
		moduleService.updateModule(module);
    }
    
    public Module getThisModule() {
		final Module module = moduleService.getByTag(HeatingModule.TAG);
		if (module == null) {
			LOGGER.log(Level.SEVERE, "System is running with inconsistent HeatingModule!");
		}
		return module;
	}

}
