package miolnet.client.thermoregulator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Properties;

public class Configuration implements Serializable {
	
	private static final long serialVersionUID = 1L;

	public enum ThermoRegulatorMode {
		MIXED, CLIENT_ONLY, STAND_ALONE
	}

	private ThermoRegulatorMode mode;
	private String serverIp;
	private String serverPort;
	private String sensorUSC;
	private float temp;
	private float difference;
	private int interval;
	private boolean enabled;
	private String gpioPin;
	private boolean gpioLowPinStateEqHeatingOn;
	
	public Configuration() {
		mode = ThermoRegulatorMode.CLIENT_ONLY;
		serverIp = "127.0.0.1";
		serverPort = "3700";
		sensorUSC = "";
		temp = 22;
		difference = 1;
		interval = 30000;
		enabled = true;
		gpioPin = "GPIO_00";
		gpioLowPinStateEqHeatingOn = true;
	}
	
	public ThermoRegulatorMode getMode() {
		return mode;
	}

	public void setMode(ThermoRegulatorMode mode) {
		this.mode = mode;
	}

	public String getServerIp() {
		return serverIp;
	}

	public void setServerIp(String serverIp) {
		if (serverIp != null) {
			this.serverIp = serverIp;
		}
	}

	public String getServerPort() {
		return serverPort;
	}

	public void setServerPort(String serverPort) {
		if (serverPort != null) {
			this.serverPort = serverPort;
		}
	}

	public String getSensorUSC() {
		return sensorUSC;
	}

	public void setSensorUSC(String sensorUSC) {
		if (sensorUSC != null) {
			this.sensorUSC = sensorUSC;
		}
	}

	public float getTemp() {
		return temp;
	}

	public void setTemp(float temp) {
		this.temp = temp;
	}

	public float getDifference() {
		return difference;
	}

	public void setDifference(float difference) {
		this.difference = difference;
	}

	public int getInterval() {
		return interval;
	}

	public void setInterval(int interval) {
		this.interval = interval;
	}

	public boolean isEnabled() {
		return enabled;
	}

	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}
	
	public String getGpioPin() {
		return gpioPin;
	}

	public void setGpioPin(String gpioPin) {
		this.gpioPin = gpioPin;
	}

	public boolean isGpioLowPinStateEqHeatingOn() {
		return gpioLowPinStateEqHeatingOn;
	}

	public void setGpioLowPinStateEqHeatingOn(boolean gpioLowPinStateEqHeatingOn) {
		this.gpioLowPinStateEqHeatingOn = gpioLowPinStateEqHeatingOn;
	}

	@Override
	public String toString() {
		return "Configuration [mode=" + mode + ", serverIp=" + serverIp + ", serverPort=" + serverPort + ", sensorUSC="
				+ sensorUSC + ", temp=" + temp + ", difference=" + difference + ", interval=" + interval + ", enabled="
				+ enabled + ", gpioPin=" + gpioPin + ", gpioLowPinStateEqHeatingOn=" + gpioLowPinStateEqHeatingOn + "]";
	}

	public static void save(final Configuration config, final String filename) throws IOException {
		final Properties prop = new Properties();
		Field[] fields = Configuration.class.getDeclaredFields();
		for (int i = 0; i < fields.length; i++) {
			if (fields[i].getName().equals("serialVersionUID")) {
				continue;
			}
			try {
				prop.setProperty(fields[i].getName(), String.valueOf(fields[i].get(config)));
			} catch (IllegalArgumentException | IllegalAccessException e) {
				e.printStackTrace();
			}
		}
		try (FileOutputStream out = new FileOutputStream(new File(filename))) {
			prop.store(out, "Miolnet ThermoRegulator config file");
		}
	}
	
	public static Configuration load(final String filename) throws FileNotFoundException, IOException {
		final Properties prop = new Properties();
		try (FileInputStream in = new FileInputStream(new File(filename))) {
			prop.load(in);
		}
		Configuration conf = new Configuration();
		String strMode = prop.getProperty("mode");
		if (strMode != null) {
			try {
				ThermoRegulatorMode mode = ThermoRegulatorMode.valueOf(strMode);
				conf.setMode(mode);
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			}
		}
		conf.setServerIp(prop.getProperty("serverIp"));
		conf.setServerPort(prop.getProperty("serverPort"));
		conf.setSensorUSC(prop.getProperty("sensorUSC"));
		String strTemp = prop.getProperty("temp");
		if (strTemp != null) {
			try {
				conf.setTemp(Float.valueOf(strTemp));
			} catch(NumberFormatException e) {
				e.printStackTrace();
			}
		}
		String strDifference = prop.getProperty("difference");
		if (strDifference != null) {
			try {
				conf.setDifference(Float.valueOf(strDifference));
			} catch(NumberFormatException e) {
				e.printStackTrace();
			}
		}
		String strInterval = prop.getProperty("interval");
		if (strInterval != null) {
			try {
				conf.setInterval(Integer.valueOf(strInterval));
			} catch(NumberFormatException e) {
				e.printStackTrace();
			}
		}
		conf.setEnabled(Boolean.valueOf(prop.getProperty("enabled")));
		conf.setGpioPin(prop.getProperty("gpioPin"));
		conf.setGpioLowPinStateEqHeatingOn(Boolean.valueOf(prop.getProperty("gpioLowPinStateEqHeatingOn")));
		return conf;
	}
	
}
