package miolnet.client.thermoregulator;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.pi4j.io.gpio.GpioController;
import com.pi4j.io.gpio.GpioFactory;
import com.pi4j.io.gpio.GpioPinDigitalOutput;
import com.pi4j.io.gpio.Pin;
import com.pi4j.io.gpio.PinState;
import com.pi4j.io.gpio.RaspiPin;

public class GpioHeatingSwitch implements HeatingSwitch {
	
	private static final Logger LOGGER = Logger.getLogger(GpioHeatingSwitch.class.getName());
	
	private boolean isLowEqOn;
	private GpioPinDigitalOutput pin;
	
	public GpioHeatingSwitch(final String gpioPin, boolean isLowEqOn) {
		LOGGER.log(Level.INFO, "Initializing GPIO interface..... ");
		final GpioController gpio = GpioFactory.getInstance();
		final GpioPinDigitalOutput pin = gpio.provisionDigitalOutputPin(getPin(gpioPin) , "Relay", isLowEqOn?PinState.HIGH:PinState.LOW);
		pin.setShutdownOptions(true, isLowEqOn?PinState.HIGH:PinState.LOW);
		this.isLowEqOn = isLowEqOn;
		this.pin = pin;
		LOGGER.log(Level.INFO, "done");
	}

	@Override
	public void turnOn() {
		if (isLowEqOn) {
			pin.low();
			LOGGER.log(Level.INFO, "Pin state LOW");
		}
		else {
			pin.high();
			LOGGER.log(Level.INFO, "Pin state HIGH");
		}
	}

	@Override
	public void turnOff() {
		if (isLowEqOn) {
			pin.high();
			LOGGER.log(Level.INFO, "Pin state HIGH");
		}
		else {
			pin.low();
			LOGGER.log(Level.INFO, "Pin state LOW");
		}
	}

	@Override
	public boolean isOn() {
		if (isLowEqOn) {
			return pin.isLow();
		}
		else {
			return pin.isHigh();
		}
	}

	@Override
	public boolean isOff() {
		if (isLowEqOn) {
			return pin.isHigh();
		}
		else {
			return pin.isLow();
		}
	}
	
	private Pin getPin(final String gpioPin) {
		switch (gpioPin) {
			case "GPIO_01": return RaspiPin.GPIO_01;
			case "GPIO_02": return RaspiPin.GPIO_02;
			case "GPIO_03": return RaspiPin.GPIO_03;
			case "GPIO_04": return RaspiPin.GPIO_04;
			case "GPIO_05": return RaspiPin.GPIO_05;
			case "GPIO_06": return RaspiPin.GPIO_06;
			case "GPIO_07": return RaspiPin.GPIO_07;
			case "GPIO_08": return RaspiPin.GPIO_08;
			case "GPIO_09": return RaspiPin.GPIO_09;
			case "GPIO_10": return RaspiPin.GPIO_10;
			case "GPIO_11": return RaspiPin.GPIO_11;
			case "GPIO_12": return RaspiPin.GPIO_12;
			case "GPIO_13": return RaspiPin.GPIO_13;
			case "GPIO_14": return RaspiPin.GPIO_14;
			case "GPIO_15": return RaspiPin.GPIO_15;
			case "GPIO_16": return RaspiPin.GPIO_16;
			case "GPIO_17": return RaspiPin.GPIO_17;
			case "GPIO_18": return RaspiPin.GPIO_18;
			case "GPIO_19": return RaspiPin.GPIO_19;
			case "GPIO_20": return RaspiPin.GPIO_20;
			case "GPIO_21": return RaspiPin.GPIO_21;
			case "GPIO_22": return RaspiPin.GPIO_22;
			case "GPIO_23": return RaspiPin.GPIO_23;
			case "GPIO_24": return RaspiPin.GPIO_24;
			case "GPIO_25": return RaspiPin.GPIO_25;
			case "GPIO_26": return RaspiPin.GPIO_26;
			case "GPIO_27": return RaspiPin.GPIO_27;
			case "GPIO_28": return RaspiPin.GPIO_28;
			case "GPIO_29": return RaspiPin.GPIO_29;
			case "GPIO_30": return RaspiPin.GPIO_30;
			case "GPIO_31": return RaspiPin.GPIO_31;
			case "GPIO_00":
			default:
		}
		return RaspiPin.GPIO_00;
	}

}
