package miolnet.client.thermoregulator;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.logging.Level;
import java.util.logging.Logger;

public class LocalLinux1wSensor implements Sensor {
	
	public static final Logger LOGGER = Logger.getLogger(LocalLinux1wSensor.class.getName());
	private static final String W1SENSORINTERFACE = "cat /sys/bus/w1/devices/%s/w1_slave";
	
	private final String usc;
	
	public LocalLinux1wSensor(final String usc) {
		this.usc = usc;
	}

	@Override
	public boolean isPresent() {
		Runtime rt = Runtime.getRuntime();
		try {
			Process p = rt.exec(String.format(W1SENSORINTERFACE, usc));
			BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
			StringBuilder fullOutputBuilder = new StringBuilder();
			String line;
			while ((line = in.readLine()) != null) {
				fullOutputBuilder.append(line);
			}
			in.close();
			String fullOutput = fullOutputBuilder.toString();
			if (!fullOutput.isEmpty()) {
				return !fullOutput.contains("No such file or directory");
			}
		} catch (IOException e) {
			LOGGER.log(Level.WARNING, e.getLocalizedMessage(), e);
		}
		return false;
	}

	@Override
	public float getValue() throws ValueReadingException {
		Runtime rt = Runtime.getRuntime();
		try {
			Process p = rt.exec(String.format(W1SENSORINTERFACE, usc));
			BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
			StringBuilder fullOutputBuilder = new StringBuilder();
			String line;
			while ((line = in.readLine()) != null) {
				fullOutputBuilder.append(line);
			}
			in.close();
			String fullOutput = fullOutputBuilder.toString();
			if (!fullOutput.isEmpty()) {
				if (fullOutput.indexOf("YES") == -1) {
					throw new ValueReadingException(usc, fullOutput + " CRC check failed!");
				}
				String value = fullOutput.split("t=")[1].trim();
				if (value != null) {
					try {
						float temp = Float.parseFloat(value);
						//temp output is in milidegrees
						temp = temp / 1000;
						return temp;
					} catch(NumberFormatException e) {
						throw new ValueReadingException(usc, fullOutput, e);
					}
				}
				throw new ValueReadingException(usc, fullOutput);
			}
		} catch (IOException e) {
			LOGGER.log(Level.WARNING, e.getLocalizedMessage(), e);
		}
		throw new ValueReadingException(usc, "");
	}
	
	public static void main(String[] args) {
		if (args.length < 1) {
			System.out.println("This test requires sensor USC as parameter!");
			return;
		}
		System.out.println("Running sensor reading test.....");
		LocalLinux1wSensor sensor = new LocalLinux1wSensor(args[0]);
		boolean sensorPresent = sensor.isPresent();
		System.out.println(String.format("Sensor present: %s", sensorPresent?"YES":"NO"));
		if (sensorPresent) {
			try {
				float sensorValue = sensor.getValue();
				System.out.println(String.format("Sensor value: %f", sensorValue));
				System.out.println("Test OK!");
				return;
			} catch (ValueReadingException e) {
				e.printStackTrace();
			}
		}
		System.out.println("Test failed!");
	}

}
