package miolnet.web.controllers;

import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import miolnet.model.Sensor;
import miolnet.model.Temperature;
import miolnet.services.SensorServiceLocal;
import miolnet.services.TemperatureServiceLocal;

@ManagedBean
@RequestScoped
public class SensorController {

	@EJB
	private SensorServiceLocal sensorService;
	
	@EJB
	private TemperatureServiceLocal tempService;
	
	private Sensor sensor;
	
	@PostConstruct
	public void init() {
		HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		String paramId = request.getParameter("id");
		
		if (paramId != null) {
			try {
				int id = Integer.parseInt(paramId);
				final Sensor s = sensorService.getById(id);
				if (s != null) {
					sensor = s;
					return;
				}
			} catch (NumberFormatException e) {
			}
		}
		sensor = new Sensor();
	}
	
	public String update() {
		final Sensor s = sensorService.getById(sensor.getId());
		s.setName(sensor.getName());
		s.setNotes(sensor.getNotes());
		sensorService.updateSensor(s);
		return "sensor.jsf?id="+sensor.getId()+"&faces-redirect=true";
	}
	
	public float getCurrentTemp(Sensor sensor) {
		return tempService.getCurrentTemperature(sensor).getTemp();
	}
	
	public float getAverageHourTemp(Sensor sensor) {
		final Date d = new Date();
		return tempService.getAverageTemperature(sensor, new Date(d.getTime()-3600000), d);
	}
	
	public Temperature getMaxDayTemp(Sensor sensor) {
		final Date d = new Date();
		return tempService.getMaxTemperature(sensor, new Date(d.getTime()-86400000), d);
	}
	
	public Temperature getMinDayTemp(Sensor sensor) {
		final Date d = new Date();
		return tempService.getMinTemperature(sensor, new Date(d.getTime()-86400000), d);
	}
	
	public List<Temperature> getDayTemperatures(Sensor sensor) {
		return tempService.getDayTemperatures(sensor);
	}

	public Sensor getSensor() {
		return sensor;
	}

	public void setSensor(Sensor sensor) {
		this.sensor = sensor;
	}
	
}
