package miolnet.web.controllers;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;

import miolnet.model.Sensor;
import miolnet.model.Setting;
import miolnet.services.SensorServiceLocal;
import miolnet.services.SettingServiceLocal;

@ManagedBean
@RequestScoped
public class SettingsController {

	@EJB
	private SettingServiceLocal settingsService;
	
	@EJB
	private SensorServiceLocal sensorService;
	
	private List<Sensor> allSensors;
	private List<Sensor> selected;
	private List<Sensor> selectedCharts;
	
	@PostConstruct
	public void init() {
		allSensors = sensorService.getAll();
		if (allSensors == null) {
			allSensors = new ArrayList<>();
		}
		List<Setting> settedDashboardSensors = settingsService.getSettingsByName("dashboardSensor");
		List<Setting> settedDashboardChartSensors = settingsService.getSettingsByName("dashboardChartSensor");
		selected = getSelectedSensorsFromSettings(settedDashboardSensors);
		selectedCharts = getSelectedSensorsFromSettings(settedDashboardChartSensors);
	}
	
	public void save() {
		saveToSettings(createStringList(selected), "dashboardSensor");
		saveToSettings(createStringList(selectedCharts), "dashboardChartSensor");
	}
	
	private List<String> createStringList(List<Sensor> selected) {
		final List<String> stringList = new ArrayList<>();
		for (final Sensor s : selected) {
			stringList.add(s.getUsc());
		}
		return stringList;
	}

	public List<Sensor> getSelectedSensorsFromSettings(final List<Setting> settings) {
		final List<Sensor> selectedSensors = new ArrayList<>();
		if (settings != null && !settings.isEmpty()) {
			for (final Setting setting : settings) {
				final Sensor s = sensorService.getByUsc(setting.getValue());
				if (s != null) {
					selectedSensors.add(s);
				}
			}
		}
		return selectedSensors;
	}
	
	private void saveToSettings(final List<String> values, final String key) {
		removeFromSettings(key);
		for (final String value : values) {
			settingsService.createSetting(key, value);
		}
	}
	
	private void removeFromSettings(final String key) {
		List<Setting> current = settingsService.getSettingsByName(key);
		for (final Setting setting : current) {
			settingsService.removeSetting(setting);
		}
	}

	public List<Sensor> getAllSensors() {
		return allSensors;
	}

	public void setAllSensors(List<Sensor> allSensors) {
		this.allSensors = allSensors;
	}

	public List<Sensor> getSelected() {
		return selected;
	}

	public void setSelected(List<Sensor> selected) {
		this.selected = selected;
	}

	public List<Sensor> getSelectedCharts() {
		return selectedCharts;
	}

	public void setSelectedCharts(List<Sensor> selectedCharts) {
		this.selectedCharts = selectedCharts;
	}
	
}
