package miolnet.web.controllers;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import miolnet.model.Role;
import miolnet.model.User;
import miolnet.services.RoleServiceLocal;
import miolnet.services.UserServiceLocal;

@ManagedBean
@RequestScoped
public class UserController {

	@EJB
	private UserServiceLocal userService;
	
	@EJB
	private RoleServiceLocal roleService;
	
	private List<Role> roles;
	
	private User user;
	private String password1;
	private String password2;
	
	public UserController() {
		// TODO Auto-generated constructor stub
	}
	
	@PostConstruct
	public void init() {
		roles = roleService.getAll();
		HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		String idParam = request.getParameter("id");
		
		if (idParam != null) {
			try {
				int id = Integer.parseInt(idParam);
				final User u = userService.getById(id);
				if (u != null) {
					user = u;
					return;
				}
			} catch(NumberFormatException e) {
			}
		}
		
		user = new User();
	}
	
	public String add() {
		User u2 = userService.getByUsername(user.getUsername());
		if (u2 != null) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Uživatelské jméno '"+user.getUsername()+"' již existuje!"));
			return null;
		}
		if (password1 == null || password1.isEmpty()) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Je vyžadováno heslo!"));
			return null;
		}
		if (!password1.equals(password2)) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Hesla se neshodují!"));
			return null;
		}
		userService.createUser(user.getUsername(), password1, user.getRole());
		return "users.jsf?faces-redirect=true";
	}
	
	public String update() {
		User u2 = userService.getByUsername(user.getUsername());
		if (u2 != null && u2.getId() != user.getId()) {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage("Uživatelské jméno '" + user.getUsername() + "' již existuje!"));
			return null;
		}
		if (password1 != null && !password1.isEmpty() && password1.equals(password2)) {
			user.setPassword(password1);
		}
		userService.updateUser(user);
		return "users.jsf?faces-redirect=true";
	}
	
	public String remove(int id) {
		User u = userService.getById(id);
		if (u == null) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Sběrná jednotka s id "+id+" neexistuje!"));
			return null;
		}
		userService.removeUser(u);
		return "users.jsf?faces-redirect=true";
	}

	public List<Role> getRoles() {
		return roles;
	}

	public void setRoles(List<Role> roles) {
		this.roles = roles;
	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public String getPassword1() {
		return password1;
	}

	public void setPassword1(String password1) {
		this.password1 = password1;
	}

	public String getPassword2() {
		return password2;
	}

	public void setPassword2(String password2) {
		this.password2 = password2;
	}
	
}
