package miolnet.web.controllers.modheating;

import java.util.Iterator;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import miolnet.model.Module;
import miolnet.model.ModuleSensor;
import miolnet.model.Sensor;
import miolnet.model.Setting;
import miolnet.modules.heating.model.SensorData.Type;
import miolnet.modules.heating.services.HeatingControlServiceLocal;
import miolnet.services.ModuleServiceLocal;
import miolnet.services.SensorServiceLocal;

@ManagedBean
@RequestScoped
public class HeatingsensorController {
	
	@EJB
	private ModuleServiceLocal moduleService;
	
	@EJB
	private HeatingControlServiceLocal controlService;
	
	@EJB
	private SensorServiceLocal sensorService;
	
	private Module heatingModule;
	private List<Sensor> sensors;
	
	private ModuleSensor s;
	private String type;

	@PostConstruct
	public void init() {
		heatingModule = moduleService.getByTag(controlService.getMTag());
		HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		String idParam = request.getParameter("id");
		
		if (idParam != null) {
			try {
				int id = Integer.parseInt(idParam);
				for (ModuleSensor msensor : heatingModule.getModuleSensors()) {
					if (msensor.getId() == id) {
						s = msensor;
						type = controlService.getSensorType(s.getSensor().getUsc())==Type.OUTDOOR?"outdoor":"indoor";
						return;
					}
				}
			} catch(NumberFormatException e) {
			}
		}
		
		s = new ModuleSensor();
		s.setPriority(5);

		sensors = sensorService.getAll();
	}
	
	public String add() {
		s.setModule(heatingModule);
		heatingModule.getModuleSensors().add(s);
		if ("outdoor".equals(type)) {
			final Setting setting = new Setting("heatingOutdoorSensorUSC", s.getSensor().getUsc());
			setting.setModule(heatingModule);
			heatingModule.getSettings().add(setting);
		}
		moduleService.updateModule(heatingModule);
		return "heating.jsf?faces-redirect=true";
	}
	
	public String update() {
		for (final ModuleSensor msensor : heatingModule.getModuleSensors()) {
			if (msensor.getId() == s.getId()) {
				final Iterator<Setting> it = heatingModule.getSettings().iterator();
				boolean present = false;
				Setting setting;
				while (it.hasNext()) {
					setting = it.next();
					if (setting != null && "heatingOutdoorSensorUSC".equals(setting.getName())
							&& setting.getValue().equals(msensor.getSensor().getUsc())) {
						if (!"outdoor".equals(type)) {
							setting.setModule(null);
							it.remove();
						}
						present = true;
						break;
					}
				}
				if ("outdoor".equals(type) && !present) {
					setting = new Setting("heatingOutdoorSensorUSC", msensor.getSensor().getUsc());
					setting.setModule(heatingModule);
					heatingModule.getSettings().add(setting);
				}

				msensor.setPriority(s.getPriority());
				break;
			}
		}
		
		moduleService.updateModule(heatingModule);
		return "heating.jsf?faces-redirect=true";
	}
	
	public String remove(int id) {
		final Iterator<ModuleSensor> it = heatingModule.getModuleSensors().iterator();
		ModuleSensor msensor;
		while (it.hasNext()) {
			msensor = it.next();
			if (msensor.getId() == id) {
				
				final Iterator<Setting> it2 = heatingModule.getSettings().iterator();
				Setting setting;
				while (it2.hasNext()) {
					setting = it2.next();
					if (setting != null && 
						"heatingOutdoorSensorUSC".equals(setting.getName()) && 
						setting.getValue().equals(msensor.getSensor().getUsc())) {
						setting.setModule(null);
						it2.remove();
					}
				}
				
				msensor.setModule(null);
				it.remove();
				moduleService.updateModule(heatingModule);
				return "heating.jsf?faces-redirect=true";
			}
		}
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Senzor modulu vytápění s id "+id+" neexistuje!"));
		return null;
	}

	public ModuleSensor getS() {
		return s;
	}

	public void setS(ModuleSensor s) {
		this.s = s;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public List<Sensor> getSensors() {
		return sensors;
	}

	public void setSensors(List<Sensor> sensors) {
		this.sensors = sensors;
	}
	
}
